# -*- coding:utf-8 -*-
#
# MIT License
#
# Copyright (c) 2020 Mattia Verga <mattia.verga@protonmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

"""A script for the new package review tickets page generation."""

from datetime import datetime
from jinja2 import Environment, FileSystemLoader
from pkg_resources import resource_filename
import click
import configparser
import sys
import time

from review_stats import log, __version__ as version
from review_stats.logging import setup_logging
from review_stats.utils import (
    bug_time, bz_connect, copy_static_content, generate_page, get_hidden, get_in_progress,
    get_reviewable, get_reviewable_epel, get_top_blockers, get_top_submitters,
    order_by_month, run_query)


@click.command()
@click.option('-c', '--config', 'configfile', type=click.File(), required=True)
@click.option('-t', '--templatedir', 'templdir', type=click.Path(exists=True),
              default=resource_filename('review_stats', 'templates/'), show_default=True)
@click.option('-s', '--staticfilesdir', 'staticdir', type=click.Path(exists=True),
              default=resource_filename('review_stats', 'static/'), show_default=True)
@click.option('-d', '--destination', 'dirname', type=click.Path(exists=True, writable=True),
              required=True)
@click.option('-v', '--verbose', 'loglevel', flag_value='INFO')
@click.option('-D', '--debug', 'loglevel', flag_value='DEBUG')
def make_page(configfile, templdir, staticdir, dirname, loglevel):
    """Builds Fedora's new package review tickets page."""
    loglevel = loglevel or 'WARNING'

    config = configparser.ConfigParser()
    config.read_file(configfile)

    setup_logging(loglevel, config)

    try:
        bz = bz_connect(config)
    except configparser.NoOptionError as ex:
        log.critical(f'Missing required value in configuration file: {ex.section}:{ex.option}')
        sys.exit(1)
    except Exception:
        log.critical('Can\'t log into Bugzilla: ', exc_info=1)
        sys.exit(1)

    try:
        bz_tickets = run_query(bz)
    except Exception:
        log.critical('Something went wrong while fetching data from Bugzilla. '
                     'I can\'t proceed further.', exc_info=1)
        sys.exit(1)

    env = Environment(loader=FileSystemLoader(templdir))
    env.globals['now'] = datetime.utcnow
    env.globals['version'] = version
    env.filters['bugtime'] = bug_time

    log.info('Starting HTML page rendering...')
    counters = {}
    t = time.time()

    # Reviewable Fedora tickets
    new_tickets = get_reviewable(bz_tickets)
    counters['reviewable'] = len(new_tickets)
    generate_page(env, 'by_month.html', dirname, 'reviewable.html',
                  page_title='New, reviewable Fedora package review tickets',
                  months=order_by_month(new_tickets),
                  count=counters['reviewable'])

    # Reviewable EPEL tickets
    new_epel_tickets = get_reviewable_epel(bz_tickets)
    counters['reviewable_epel'] = len(new_epel_tickets)
    generate_page(env, 'by_month.html', dirname, 'reviewable_epel.html',
                  page_title='New, reviewable EPEL only package review tickets',
                  months=order_by_month(new_epel_tickets),
                  count=counters['reviewable_epel'])

    # Reviews in progress
    in_progress_tickets = get_in_progress(bz_tickets)
    counters['in_progress'] = len(in_progress_tickets)
    generate_page(env, 'in_progress.html', dirname, 'in_progress.html',
                  page_title='All tickets currently under review',
                  tickets=sorted(in_progress_tickets, key=lambda t: t.last_modified),
                  count=counters['in_progress'])

    # Hidden tickets
    hidden_tickets = get_hidden(bz_tickets)
    counters['hidden'] = len(hidden_tickets)
    generate_page(env, 'hidden.html', dirname, 'hidden.html',
                  page_title='All tickets hidden from the main review queues',
                  months=order_by_month(hidden_tickets),
                  count=counters['hidden'])

    # Needsponsor tickets
    # Some of them are in the in progress queue, but still waiting for a sponsor
    needsponsor_tickets = [tk for tk in bz_tickets if not tk.hidden and tk.needsponsor]
    counters['needsponsor'] = len(needsponsor_tickets)
    generate_page(env, 'plain.html', dirname, 'needsponsor.html',
                  page_title='All review tickets where a sponsor is required',
                  tickets=sorted(needsponsor_tickets, key=lambda t: t.last_modified),
                  count=counters['needsponsor'])

    # Trivial tickets
    trivial_tickets = [tk for tk in get_reviewable(bz_tickets) + get_reviewable_epel(bz_tickets)
                       if not tk.hidden and tk.trivial]
    counters['trivial'] = len(trivial_tickets)
    generate_page(env, 'by_month.html', dirname, 'trivial.html',
                  page_title='All tickets marked as trivial',
                  months=order_by_month(trivial_tickets),
                  count=counters['trivial'])

    # Top blockers
    top_blockers = get_top_blockers(bz_tickets)
    counters['blockers'] = len(top_blockers)
    generate_page(env, 'blockers.html', dirname, 'blockers.html',
                  page_title='All tickets which block other tickets',
                  blockers=top_blockers,
                  count=counters['blockers'])

    # Top submitters
    top_submitters = get_top_submitters(bz_tickets)
    counters['submitters'] = len(top_submitters)
    generate_page(env, 'submitters.html', dirname, 'submitters.html',
                  page_title='All review tickets sorted by submitter',
                  submitters=top_submitters,
                  count=counters['submitters'])

    # Finally, generate the index page
    generate_page(env, 'index.html', dirname, 'index.html',
                  counters=counters)
    log.info(f'Done HTML rendering, took {time.time() - t:.2f}s.')

    copy_static_content(staticdir, dirname)


if __name__ == '__main__':
    make_page()
