# -*- coding:utf-8 -*-
#
# MIT License
#
# Copyright (c) 2020 Mattia Verga <mattia.verga@protonmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

"""Utils for review_stats.py script."""

from datetime import datetime
import bugzilla
import os
import shutil
import tempfile
import time

from . import log, trackers
from .models import ReviewTicket


def bz_connect(config):
    bz_url = config.get('review-stats', 'url', fallback='https://bugzilla.redhat.com/xmlrpc.cgi')
    log.debug(f'Using Bugzilla instance from "{bz_url}".')

    if 'bugzilla_api_key' in config['review-stats']:
        bz_api = config.get('review-stats', 'bugzilla_api_key')
        log.debug('Authenticating with API token.')
        bz = bugzilla.Bugzilla(url=bz_url, api_key=bz_api)
    else:
        bz_username = config.get('review-stats', 'username')
        bz_password = config.get('review-stats', 'password')
        log.debug('Authenticating with username and password.')
        bz = bugzilla.Bugzilla(url=bz_url, user=bz_username, password=bz_password)
    return bz


def run_query(bz):
    """Get data from Bugzilla."""
    querydata = {
        'include_fields': [
            'id', 'creation_time', 'last_change_time', 'bug_severity', 'alias', 'assigned_to',
            'product', 'creator', 'creator_id', 'status', 'resolution', 'component', 'blocks',
            'depends_on', 'summary', 'whiteboard', 'flags',
            ],
        'bug_status': ['NEW', 'ASSIGNED', 'MODIFIED'],
        'product': ['Fedora', 'Fedora EPEL'],
        'component': ['Package Review'],
        }

    log.info('Quering Bugzilla for the review tickets list...')
    t = time.time()
    bugs = [b for b in bz.query(querydata) if b.id not in trackers]
    log.info(f'Done, retrieved {len(bugs)} tickets, took {time.time() - t:.2f}s.')

    # Retrieve information about blockers
    blockers = set()
    for bug in bugs:
        if bug.depends_on:
            blockers.update(bug.depends_on)
    log.info('Quering Bugzilla for the blockers list...')
    t = time.time()
    blockers = [b for b in bz.query(bz.build_query(bug_id=list(blockers)))]
    log.info(f'Done, retrieved information for {len(blockers)} blockers, '
             f'took {time.time() - t:.2f}s.')

    return [ReviewTicket(bug, blockers) for bug in bugs]


def bug_time(value, format='%Y-%m-%d %H:%M:%S'):
    """Convert bug time in a readable string."""

    bugtime = datetime.strptime(value, "%Y%m%dT%H:%M:%S")
    return bugtime.strftime(format)


def get_hidden(tklist):
    """Return all hidden tickets."""

    tklist = [tk for tk in tklist if tk.hidden]
    return sorted(tklist, key=lambda b: b.bug.id)


def get_in_progress(tklist):
    """Return a list of tickets which review is in progress.

    Gets all tickets with `fedora-review` flag set OR assigned.
    Ideally, they should have both. We will mark inconsistencies
    in the output webpage.
    """

    tklist = [tk for tk in tklist if tk.review_flag_status
              or tk.really_assigned]
    return sorted(tklist, key=lambda b: b.bug.id)


def get_reviewable(tklist):
    """Return a list of reviewable tickets."""

    tklist = [tk for tk in tklist if tk.review_flag_status == ''
              and not tk.really_assigned
              and not tk.hidden
              and not tk.bug.product == 'Fedora EPEL']
    return sorted(tklist, key=lambda b: b.bug.id)


def get_reviewable_epel(tklist):
    """Return a list of EPEL only reviewable tickets."""

    tklist = [tk for tk in tklist if tk.review_flag_status == ''
              and not tk.really_assigned
              and not tk.hidden
              and tk.bug.product == 'Fedora EPEL']
    return sorted(tklist, key=lambda b: b.bug.id)


def get_top_blockers(tklist):
    """Return a dictionary with counters about open blockers."""

    blockers = dict()
    for tk in tklist:
        for blocker in tk.blocked:
            if blocker not in blockers:
                blockers[blocker] = [tk]
            else:
                blockers[blocker].append(tk)
    return {k: v for k, v in sorted(blockers.items(),
                                    key=lambda item: len(item[1]), reverse=True)}


def get_top_submitters(tklist):
    """Return a dictionary with counters about submitters."""

    submitters = dict()
    for tk in tklist:
        reporter = tk.bug.reporter
        if reporter not in submitters:
            submitters[reporter] = [tk]
        else:
            submitters[reporter].append(tk)
    return {k: v for k, v in sorted(submitters.items(),
                                    key=lambda item: len(item[1]), reverse=True)}


def generate_page(env, tmplt_name, destdir, page_name, *args, **kwargs):
    """Generate HTML page."""
    filename = os.path.join(destdir, page_name)
    try:
        template = env.get_template(tmplt_name)
        with tempfile.NamedTemporaryFile(mode='w+', delete=False) as ft:
            ft.write(template.render(*args, **kwargs))
            ft.close()
            shutil.copy2(ft.name, filename)
    except Exception:
        log.error(f'Error while generating page {page_name}:', exc_info=1)


def copy_static_content(staticdir, destdir):
    """Copy static content to the output dir."""
    destination_staticdir = os.path.join(destdir, 'static/')
    try:
        if os.path.exists(destination_staticdir):
            shutil.rmtree(destination_staticdir)
        shutil.copytree(staticdir, destination_staticdir)
    except Exception:
        log.error(f'Error while copying static files to destination:', exc_info=1)


def order_by_month(tklist):
    """Return a list with tickets listed by month."""

    by_month = {}
    for tk in tklist:
        k = tk.created.strftime('%B %Y')
        if k not in by_month:
            by_month[k] = [tk]
        else:
            by_month[k].append(tk)
    by_month_list = [(k, v) for k, v in by_month.items()]
    return sorted(by_month_list, key=lambda d: datetime.strptime(d[0], '%B %Y'))
